<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron" defaultPhase="check">
    <!-- Define an empty phase with no active patterns -->
    <phase id="empty"/>
    
    <!-- Define a phase named "check" with several active patterns -->
    <phase id="check">
        <active pattern="line"/>
        <active pattern="lineGroup"/>
        <active pattern="notimplemented"/>
        <active pattern="headers"/>
        <active pattern="deprecatedtable"/>
        <active pattern="book"/>
        <active pattern="contentType"/>
    </phase>
    
    <!-- Define a pattern for validating "line" elements -->
    <pattern id="line">
        <title>
            <dir value="ltr">Well-formed line elements</dir>
        </title>
        <rule context="line">
            <!-- Ensure there is at most one "langData" element within a "line" element -->
            <assert test="count(langData) &lt;= 1">Too many langData elements in this line element.  Use only one langData in order to produce properly aligned output.</assert>
            <!-- Ensure there is at most one "gloss" element within a "line" element -->
            <assert test="count(gloss) &lt;= 1">Too many gloss elements in this line element.  Use only one gloss in order to produce properly aligned output.</assert>
            <!-- Report if "object" element is used within "langData" -->
            <report test="langData/object">Using an object element within a langData element here will not produce the correct output.  Use wrd elements within this line element instead.  See section 5.3.1.2 "Word-aligned, marking certain words or morphemes" in the XLingPaper user documentation.</report>
            <!-- Report if "endnote" element is used within "langData" -->
            <report test="langData/endnote">Using an endnote element within a langData element here will not produce the correct output.  Use wrd elements within this line element instead.  See section 5.3.1.2 "Word-aligned, marking certain words or morphemes" in the XLingPaper user documentation.</report>
            <!-- Report if "object" element is used within "gloss" -->
            <report test="gloss/object">Using an object element within a gloss element here will not produce the correct output.  Use wrd elements within this line element instead.  See section 5.3.1.2 "Word-aligned, marking certain words or morphemes" in the XLingPaper user documentation.</report>
            <!-- Report if "abbrRef" element is used within "gloss" -->
            <report test="gloss/abbrRef">Using an abbrRef element within a gloss element here will not produce the correct output.  Use wrd elements within this line element instead.  See section 5.3.1.2 "Word-aligned, marking certain words or morphemes" in the XLingPaper user documentation.</report>
            <!-- Report if "endnote" element is used within "gloss" -->
            <report test="gloss/endnote">Using an endnote element within a gloss element here will not produce the correct output.  Use wrd elements within this line element instead.  See section 5.3.1.2 "Word-aligned, marking certain words or morphemes" in the XLingPaper user documentation.</report>
        </rule>
    </pattern>
    
    <!-- Define a pattern for validating "lineGroup" elements -->
    <pattern id="lineGroup">
        <title>
            <dir value="ltr">Consistent line elements within a lineGroup</dir>
        </title>
        <rule context="lineGroup">
            <!-- Report if some "line" elements use "wrd" elements and some do not within the same "lineGroup" -->
            <report test="line[count(wrd) &gt; 0] and line[count(wrd)=0]">Some of the line elements in this lineGroup have been converted to use wrd elements and some have not.  All line elements within a lineGroup should be the same: either all using wrd or all not using wrd.  The formatted output may leave out some lines.  Please use the XLingPaper menu item / 'Convert interlinear line to wrd elements' for all lines in this lineGroup.</report>
        </rule>
        <rule context="line">
            <!-- Report if the number of "wrd" elements in a "line" differs from its preceding sibling "line" -->
            <report test="preceding-sibling::line[1] and count(preceding-sibling::line[1]/wrd[not(exampleRef)]) != count(wrd)">This line has a different number of wrd elements in it than its immediately preceding line.  The alignment will probably be incorrect.</report>
        </rule>
    </pattern>
    
    <!-- Define a pattern for checking not yet implemented elements -->
    <pattern id="notimplemented">
        <title>
            <dir value="ltr">Check for yet-to-be-implemented elements</dir>
        </title>
        <rule context="conflatedLine">
            <!-- Report if "conflatedLine" element is used -->
            <report test=".">The conflatedLing element has no been implemented fully yet.  Please do not use it.</report>
        </rule>
        <rule context="lineSet">
            <!-- Report if "lineSet" element is used -->
            <report test=".">The lineSet element has no been implemented fully yet.  Please do not use it.</report>
        </rule>
        <rule context="lineSetRow">
            <!-- Report if "lineSetRow" element is used -->
            <report test=".">The lineSetRow element has no been implemented fully yet.  Please do not use it.</report>
        </rule>
    </pattern>
    
    <!-- Define a pattern for checking table headers -->
    <pattern id="headers">
        <title>
            <dir value="ltr">Check for partial table header rows</dir>
        </title>
        <rule context="table">
            <!-- Report if the first row of a table has both "th" and "td" cells -->
            <report test="not(ancestor::table) and count(tr) &gt; 5 and tr[1]/th and tr[1]/td">Warning: the first row of this table has some th cells , but it also has some td cells.  If the table goes beyond a page, no header will be repeated.  To fix this, convert the td cells to th cells (and use object elements to get the fomatting you want). </report>
        </rule>
    </pattern>
    
    <!-- Define a pattern for checking deprecated table elements -->
    <pattern id="deprecatedtable">
        <title>
            <dir value="ltr">Check for deprecated table elements</dir>
        </title>
        <rule context="col">
            <!-- Report if "col" element is used -->
            <report test=".">Warning: the col element is deprecated.  Please replace it with a td element. </report>
        </rule>
        <rule context="headerCol">
            <!-- Report if "headerCol" element is used -->
            <report test=".">Warning: the headerCol element is deprecated.  Please replace it with a th element. </report>
        </rule>
        <rule context="headerRow">
            <!-- Report if "headerRow" element is used -->
            <report test=".">Warning: the headerRow element is deprecated.  Please replace it with a tr element. </report>
        </rule>
        <rule context="row">
            <!-- Report if "row" element is used -->
            <report test=".">Warning: the row element is deprecated.  Please replace it with a tr element. </report>
        </rule>
    </pattern>
    
    <!-- Define a pattern for checking book elements with improper header and footer layouts -->
    <pattern id="book">
        <title>
            <dir value="ltr">Check for book with improper header and footer layouts</dir>
        </title>
        <rule context="publisherStyleSheet">
            <!-- Report if a book is using a paper-oriented publisher style sheet -->
            <report test="//chapter and pageLayout/headerFooterPageStyles">Warning: this is a book but it is trying to use a paper-oriented publisher style sheet (i.e. it has the header and footer layout information under page layout instead of under both front matter layout and body layout).  The XeLaTex way of producing PDF will fail.  Please associate this document with a *book* publisher style sheet..</report>
            <!-- Report if a book is missing header and footer layout information for titles under front matter layout -->
            <report test="//chapter and frontMatterLayout[not(titleHeaderFooterPageStyles)]">Warning: this is a book but it is trying to use a publisher style sheet that does not have header and footer layout information for titles under the front matter layout.  The XeLaTex way of producing PDF will fail or not be correct.  Please fix this by adding the correct header and footer layout information or by associating this document with a book publisher style sheet..</report>
            <!-- Report if a book is missing header and footer layout information for non-title front matter pages under front matter layout -->
            <report test="//chapter and frontMatterLayout[not(headerFooterPageStyles)]">Warning: this is a book but it is trying to use a publisher style sheet that does not have header and footer layout information for non-title front matter pages under the front matter layout.  The XeLaTex way of producing PDF will fail or not be correct.  Please fix this by adding the correct header and footer layout information or by associating this document with a book publisher style sheet..</report>
            <!-- Report if a book is missing header and footer layout information under body layout -->
            <report test="//chapter and bodyLayout[not(headerFooterPageStyles)]">Warning: this is a book but it is trying to use a publisher style sheet that does not have header and footer layout information under the body layout.  The XeLaTex way of producing PDF will fail or not be correct.  Please fix this by adding the correct header and footer layout information or by associating this document with a book publisher style sheet..</report>
        </rule>
    </pattern>
    
    <!-- Define a pattern for checking non-excluded reference elements to elements being excluded via a contentType -->
    <pattern id="contentType">
        <title>
            <dir value="ltr">Check for non-excluded reference elements to elements being excluded via a contentType.</dir>
        </title>
        <rule context="exampleRef[not(ancestor::*[string-length(@contentType)!=0])]">
            <!-- Report if an "exampleRef" refers to an "example" that may be excluded via a contentType -->
            <report test="@num=//example/@num[string-length(../@contentType)!=0 or ancestor::*[string-length(@contentType)!=0]] and string-length(@contentType)=0">There is an exampleRef which refers to an example which may be excluded in the output (via a contentType) and yet the exampleRef itself is always present.  Please make sure that both the example and its exampleRef are excluded in the same situations.</report>
        </rule>
        <rule context="appendixRef[not(ancestor::*[string-length(@contentType)!=0])]">
            <!-- Report if an "appendixRef" refers to an "appendix" that may be excluded via a contentType -->
            <report test="@app=//appendix/@id[string-length(../@contentType)!=0 or ancestor::*[string-length(@contentType)!=0]] and string-length(@contentType)=0">There is an appendixRef which refers to an appendix which may be excluded in the output (via a contentType) and yet the appendixRef itself is always present.  Please make sure that both the appendix and its appendixRef are excluded in the same situations.</report>
        </rule>
        <rule context="interlinearRef[not(ancestor::*[string-length(@contentType)!=0])]">
            <!-- Report if an "interlinearRef" refers to an "interlinear" that may be excluded via a contentType -->
            <report test="@textref=//interlinear/@text[string-length(../@contentType)!=0 or ancestor::*[string-length(@contentType)!=0]]">There is an interlinearRef which refers to an interlinear in an interlinear text which may be excluded in the output (via a contentType) and yet the interlinearRef itself is always present.  Please make sure that both the interlinear and its interlinearRef are excluded in the same situations.</report>
        </rule>
        <rule context="interlinearRefCitation[not(ancestor::*[string-length(@contentType)!=0])]">
            <!-- Report if an "interlinearRefCitation" refers to an "interlinear" that may be excluded via a contentType -->
            <report test="@textref=//interlinear/@text[string-length(../@contentType)!=0 or ancestor::*[string-length(@contentType]!=0]]">There is an interlinearRefCitation which refers to an interlinear in an interlinear text which may be excluded in the output (via a contentType) and yet the interlinearRefCitation itself is always present.  Please make sure that both the interlinear and its interlinearRefCitation are excluded in the same situations.</report>
        </rule>
        <rule context="figureRef[not(ancestor::*[string-length(@contentType)!=0])]">
            <!-- Report if a "figureRef" refers to a "figure" that may be excluded via a contentType -->
            <report test="@figure=//figure/@id[string-length(../@contentType)!=0 or ancestor::*[string-length(@contentType)!=0]] and string-length(@contentType)=0">There is a figureRef which refers to a figure which may be excluded in the output (via a contentType) and yet the figureRef itself is always present.  Please make sure that both the figure and its figureRef are excluded in the same situations.</report>
        </rule>
        <rule context="tablenumberedRef[not(ancestor::*[string-length(@contentType]!=0])]">
            <!-- Report if a "tablenumberedRef" refers to a "tablenumbered" that may be excluded via a contentType -->
            <report test="@table=//tablenumbered/@id[string-length(../@contentType)!=0 or ancestor::*[string-length(@contentType)!=0]] and string-length(@contentType)=0">There is a tablenumberedRef which refers to a tablenumbered which may be excluded in the output (via a contentType) and yet the tablenumberedRef itself is always present.  Please make sure that both the tablenumbered and its tablenumberedRef are excluded in the same situations.</report>
        </rule>
        <rule context="abbrRef[not(ancestor::*[string-length(@contentType)!=0])]">
            <!-- Report if there are "abbrRef" elements present when "abbreviationsShownHere" may be excluded via a contentType -->
            <report test="//abbreviationsShownHere[ancestor::*[string-length(@contentType)!=0]]">There is an abbreviationsShownHere element which may be excluded in the output (via a contentType) and yet there are one or more abbrRef elements which are always present.  Please make sure that both the abbreviationsShownHere and all abbrRefs are excluded in the same situations.</report>
        </rule>
        <rule context="genericRef[not(ancestor::*[string-length(@contentType)!=0])]">
            <!-- Report if a "genericRef" refers to a "genericTarget" that may be excluded via a contentType -->
            <report test="@gref=//genericTarget/@id[string-length(../@contentType)!=0 or ancestor::*[string-length(@contentType)!=0]] and string-length(@contentType)=0">There is a genericRef which refers to a genericTarget which may be excluded in the output (via a contentType) and yet the genericRef itself is always present.  Please make sure that both the genericTarget and its genericRef are excluded in the same situations.</report>
        </rule>
        <rule context="endnoteRef[not(ancestor::*[string-length(@contentType)!=0])]">
            <!-- Report if an "endnoteRef" refers to an "endnote" that may be excluded via a contentType -->
            <report test="@note=//endnote/@id[string-length(../@contentType)!=0 or ancestor::*[string-length(@contentType)!=0]]">There is an endnoteRef which refers to an endnote which may be excluded in the output (via a contentType) and yet the endnoteRef itself is always present.  Please make sure that both the endnote and its endnoteRef are excluded in the same situations.</report>
        </rule>
    </pattern>
</schema>
