<?xml version="1.0"?>
<xsl:stylesheet version="3.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:java="http://xml.apache.org/xslt/java">
    
    <!-- Define the output method as XML with UTF-8 encoding and indentation -->
    <xsl:output method="xml" encoding="UTF-8" indent="true"/>
    
    <!-- Template to match the root element /sqlMapConfig -->
    <xsl:template match="/sqlMapConfig">
        <!-- Add a comment with the current date and time -->
        <xsl:comment>Converted at: <xsl:value-of select="current-dateTime()"/> </xsl:comment>
        <!-- Output the DOCTYPE declaration for MyBatis configuration -->
        <xsl:text disable-output-escaping="yes"><![CDATA[
            <!DOCTYPE configuration PUBLIC "-//mybatis.org//DTD SQL Map Config 3.0//EN"
            "http://mybatis.org/dtd/mybatis-3-config.dtd">
            ]]></xsl:text>
        
        <!-- Create the root element <configuration> -->
        <xsl:element name="configuration">
            <!-- Apply templates to the <settings> element -->
            <xsl:apply-templates select="/sqlMapConfig/settings"/>
            <xsl:text>
            </xsl:text>
            <!-- Create the <typeAliases> element -->
            <xsl:element name="typeAliases">
                <xsl:text>
                </xsl:text>
                <!-- Loop through each <typeAlias> element and copy it -->
                <xsl:for-each select="/sqlMapConfig/typeAlias">
                    <xsl:copy-of select="."/>
                    <xsl:text>
                    </xsl:text>
                </xsl:for-each>
            </xsl:element>
            <xsl:text>
                
            </xsl:text>
            <!-- Create the <typeHandlers> element -->
            <xsl:element name="typeHandlers">
                <xsl:text>
                </xsl:text>
                <!-- Loop through each <typeHandler> element and create a new <typeHandler> element -->
                <xsl:for-each select="/sqlMapConfig/typeHandler">
                    <xsl:element name="typeHandler">
                        <!-- Copy attributes from the original <typeHandler> element -->
                        <xsl:attribute name="javaType">
                            <xsl:value-of select="@javaType"/>
                        </xsl:attribute>
                        <xsl:attribute name="jdbcType">
                            <xsl:value-of select="@jdbcType"/>
                        </xsl:attribute>
                        <xsl:attribute name="handler">
                            <xsl:value-of select="@callback"/>
                        </xsl:attribute>
                    </xsl:element>
                    <xsl:text>
                    </xsl:text>
                </xsl:for-each>
            </xsl:element>
            <xsl:text>
                
            </xsl:text>
            
            <!-- Conditionally create the <objectFactory> element if <resultObjectFactory> exists -->
            <xsl:if test="resultObjectFactory">
                <xsl:element name="objectFactory">
                    <xsl:attribute name="type">
                        <xsl:value-of select="resultObjectFactory/@type"/>
                    </xsl:attribute>
                </xsl:element>
                <xsl:text>
                    
                </xsl:text>
            </xsl:if>
            
            <!-- Conditionally create the <environments> element if <transactionManager> exists -->
            <xsl:if test="transactionManager">
                <xsl:element name="environments">
                    <xsl:attribute name="default">default</xsl:attribute>
                    <xsl:text>
                    </xsl:text>
                    <xsl:element name="environment">
                        <xsl:attribute name="id">default</xsl:attribute>
                        <xsl:text>
                        </xsl:text>
                        <xsl:element name="transactionManager">
                            <xsl:attribute name="type">
                                <xsl:value-of select="transactionManager/@type"/>
                            </xsl:attribute>
                            <!-- Loop through each <property> element inside <transactionManager> and copy it -->
                            <xsl:for-each select="transactionManager/property">
                                <xsl:text>
                                </xsl:text>
                                <xsl:copy-of select="."/>
                            </xsl:for-each>
                            <xsl:text>
                            </xsl:text>
                        </xsl:element>
                        <xsl:text>
                        </xsl:text>
                        <!-- Conditionally create the <dataSource> element if it exists inside <transactionManager> -->
                        <xsl:if test="transactionManager/dataSource">
                            <xsl:text>
                            </xsl:text>
                            <xsl:element name="dataSource">
                                <xsl:attribute name="type">
                                    <xsl:value-of select="transactionManager/dataSource/@type"/>
                                </xsl:attribute>
                                <!-- Loop through each <property> element inside <dataSource> and copy it -->
                                <xsl:for-each select="transactionManager/dataSource/property">
                                    <xsl:text>
                                    </xsl:text>
                                    <xsl:copy-of select="."/>
                                </xsl:for-each>
                                <xsl:text>
                                </xsl:text>
                            </xsl:element>
                            <xsl:text>
                            </xsl:text>
                        </xsl:if>
                    </xsl:element>
                    <xsl:text>
                    </xsl:text>
                </xsl:element>
                <xsl:text>
                    
                </xsl:text>
            </xsl:if>
            
            <!-- Create the <mappers> element -->
            <xsl:element name="mappers">
                <xsl:text>
                </xsl:text>
                <!-- Loop through each <sqlMap> element and create a new <mapper> element -->
                <xsl:for-each select="/sqlMapConfig/sqlMap">
                    <xsl:element name="mapper">
                        <!-- Copy all attributes from the original <sqlMap> element -->
                        <xsl:copy-of select="@*"/>
                    </xsl:element>
                    <xsl:text>
                    </xsl:text>
                </xsl:for-each>
            </xsl:element>
            <xsl:text>
            </xsl:text>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to match the <settings> element inside /sqlMapConfig -->
    <xsl:template match="/sqlMapConfig/settings">
        <!-- Create the <settings> element -->
        <xsl:element name="settings">
            <xsl:text>
            </xsl:text>
            <!-- Conditionally create <setting> elements based on the attributes of <settings> -->
            <xsl:if test="@cacheModelsEnabled">
                <xsl:element name="setting">
                    <xsl:attribute name="name">cacheEnabled</xsl:attribute>
                    <xsl:attribute name="value">
                        <xsl:value-of select="@cacheModelsEnabled"/>
                    </xsl:attribute>
                </xsl:element>
                <xsl:text>
                </xsl:text>
            </xsl:if>
            <xsl:if test="@statementCachingEnabled = 'true'">
                <xsl:element name="setting">
                    <xsl:attribute name="name">defaultExecutorType</xsl:attribute>
                    <xsl:attribute name="value">REUSE</xsl:attribute>
                </xsl:element>
                <xsl:text>
                </xsl:text>
            </xsl:if>
            <xsl:if test="@lazyLoadingEnabled">
                <xsl:element name="setting">
                    <xsl:attribute name="name">lazyLoadingEnabled</xsl:attribute>
                    <xsl:attribute name="value">
                        <xsl:value-of select="@lazyLoadingEnabled"/>
                    </xsl:attribute>
                </xsl:element>
                <xsl:text>
                </xsl:text>
            </xsl:if>
            <xsl:if test="@defaultStatementTimeout">
                <xsl:element name="setting">
                    <xsl:attribute name="name">defaultStatementTimeout</xsl:attribute>
                    <xsl:attribute name="value">
                        <xsl:value-of select="@defaultStatementTimeout"/>
                    </xsl:attribute>
                </xsl:element>
                <xsl:text>
                </xsl:text>
            </xsl:if>
            <xsl:if test="@forceMultipleResultSetSupport">
                <xsl:element name="setting">
                    <xsl:attribute name="name">multipleResultSetsEnabled</xsl:attribute>
                    <xsl:attribute name="value">
                        <xsl:value-of select="@forceMultipleResultSetSupport"/>
                    </xsl:attribute>
                </xsl:element>
                <xsl:text>
                </xsl:text>
            </xsl:if>
            <xsl:if test="@useColumnLabel">
                <xsl:element name="setting">
                    <xsl:attribute name="name">useColumnLabel</xsl:attribute>
                    <xsl:attribute name="value">
                        <xsl:value-of select="@useColumnLabel"/>
                    </xsl:attribute>
                </xsl:element>
                <xsl:text>
                </xsl:text>
            </xsl:if>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to match the root element /sqlMap -->
    <xsl:template match="/sqlMap">
        <!-- Add a comment with the current date and time -->
        <xsl:comment>Converted at: <xsl:value-of select="current-dateTime()"/> </xsl:comment>
        <!-- Output the DOCTYPE declaration for MyBatis mapper -->
        <xsl:text disable-output-escaping="yes"><![CDATA[
            <!DOCTYPE mapper PUBLIC "-//mybatis.org//DTD Mapper 3.0//EN" 
            "http://mybatis.org/dtd/mybatis-3-mapper.dtd">
            ]]></xsl:text>
        <!-- Create the root element <mapper> -->
        <xsl:element name="mapper">
            <!-- Copy the namespace attribute from the original <sqlMap> element -->
            <xsl:attribute name="namespace">
                <xsl:value-of select="@namespace"/>
            </xsl:attribute>
            <!-- Apply templates to the child elements of <sqlMap> -->
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to match the <select> element inside /sqlMap -->
    <xsl:template match="/sqlMap/select">
        <!-- Create the <select> element -->
        <xsl:element name="select">
            <!-- Copy attributes from the original <select> element -->
            <xsl:attribute name="id">
                <xsl:value-of select="@id"/>
            </xsl:attribute>
            <xsl:if test="@parameterClass">
                <xsl:attribute name="parameterType">
                    <xsl:value-of select="@parameterClass"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@parameterMap">
                <xsl:attribute name="parameterMap">
                    <xsl:value-of select="@parameterMap"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@resultClass">
                <xsl:attribute name="resultType">
                    <xsl:value-of select="@resultClass"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@resultMap">
                <xsl:attribute name="resultMap">
                    <xsl:value-of select="@resultMap"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@fetchSize">
                <xsl:attribute name="fetchSize">
                    <xsl:value-of select="@fetchSize"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@resultSetType">
                <xsl:attribute name="resultSetType">
                    <xsl:value-of select="@resultSetType"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@timeout">
                <xsl:attribute name="timeout">
                    <xsl:value-of select="@timeout"/>
                </xsl:attribute>
            </xsl:if>
            <!-- Apply templates to the child elements of <select> -->
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to match the <insert> element inside /sqlMap -->
    <xsl:template match="/sqlMap/insert">
        <!-- Create the <insert> element -->
        <xsl:element name="insert">
            <!-- Copy attributes from the original <insert> element -->
            <xsl:attribute name="id">
                <xsl:value-of select="@id"/>
            </xsl:attribute>
            <xsl:if test="@parameterClass">
                <xsl:attribute name="parameterType">
                    <xsl:value-of select="@parameterClass"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@parameterMap">
                <xsl:attribute name="parameterMap">
                    <xsl:value-of select="@parameterMap"/>
                </xsl:attribute>
            </xsl:if>
            <!-- Apply templates to the child elements of <insert> -->
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to match the <update> element inside /sqlMap -->
    <xsl:template match="/sqlMap/update">
        <!-- Create the <update> element -->
        <xsl:element name="update">
            <!-- Copy attributes from the original <update> element -->
            <xsl:attribute name="id">
                <xsl:value-of select="@id"/>
            </xsl:attribute>
            <xsl:if test="@parameterClass">
                <xsl:attribute name="parameterType">
                    <xsl:value-of select="@parameterClass"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@parameterMap">
                <xsl:attribute name="parameterMap">
                    <xsl:value-of select="@parameterMap"/>
                </xsl:attribute>
            </xsl:if>
            <!-- Apply templates to the child elements of <update> -->
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to match the <procedure> element inside /sqlMap -->
    <xsl:template match="/sqlMap/procedure">
        <!-- Create the <update> element with statementType="CALLABLE" -->
        <xsl:element name="update">
            <!-- Copy attributes from the original <procedure> element -->
            <xsl:attribute name="id">
                <xsl:value-of select="@id"/>
            </xsl:attribute>
            <xsl:if test="@parameterClass">
                <xsl:attribute name="parameterType">
                    <xsl:value-of select="@parameterClass"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@parameterMap">
                <xsl:attribute name="parameterMap">
                    <xsl:value-of select="@parameterMap"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:attribute name="statementType">CALLABLE</xsl:attribute>
            <!-- Apply templates to the child elements of <procedure> -->
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to match the <delete> element inside /sqlMap -->
    <xsl:template match="/sqlMap/delete">
        <!-- Create the <delete> element -->
        <xsl:element name="delete">
            <!-- Copy attributes from the original <delete> element -->
            <xsl:attribute name="id">
                <xsl:value-of select="@id"/>
            </xsl:attribute>
            <xsl:if test="@parameterClass">
                <xsl:attribute name="parameterType">
                    <xsl:value-of select="@parameterClass"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@parameterMap">
                <xsl:attribute name="parameterMap">
                    <xsl:value-of select="@parameterMap"/>
                </xsl:attribute>
            </xsl:if>
            <!-- Apply templates to the child elements of <delete> -->
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to match the <resultMap> element inside /sqlMap -->
    <xsl:template match="/sqlMap/resultMap">
        <!-- Create the <resultMap> element -->
        <xsl:element name="resultMap">
            <!-- Copy attributes from the original <resultMap> element -->
            <xsl:attribute name="id">
                <xsl:value-of select="@id"/>
            </xsl:attribute>
            <xsl:if test="@class">
                <xsl:attribute name="type">
                    <xsl:value-of select="@class"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="@extends">
                <xsl:attribute name="extends">
                    <xsl:value-of select="@extends"/>
                </xsl:attribute>
            </xsl:if>
            <!-- Apply templates to the child elements of <resultMap> -->
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

	<xsl:template match="/sqlMap/resultMap/discriminator">
		<!-- Create the <discriminator> element -->
		<xsl:element name="discriminator">
			<!-- Copy the javaType attribute from the original <discriminator> element -->
			<xsl:attribute name="javaType">
				<xsl:value-of select="@javaType"/>
			</xsl:attribute>
			<!-- Conditionally copy the column attribute if it exists -->
			<xsl:if test="@column">
				<xsl:attribute name="column">
					<xsl:value-of select="@column"/>
				</xsl:attribute>
			</xsl:if>
			<!-- Conditionally copy the jdbcType attribute if it exists -->
			<xsl:if test="@jdbcType">
				<xsl:attribute name="jdbcType">
					<xsl:value-of select="@jdbcType"/>
				</xsl:attribute>
			</xsl:if>
			<!-- Conditionally copy the typeHandler attribute if it exists -->
			<xsl:if test="@typeHandler">
				<xsl:attribute name="typeHandler">
					<xsl:value-of select="@typeHandler"/>
				</xsl:attribute>
			</xsl:if>
			<!-- Apply templates to the child elements of <discriminator> -->
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="/sqlMap/resultMap/discriminator/subMap">
		<!-- Create the <case> element inside <discriminator> -->
		<xsl:element name="case">
			<!-- Copy the value attribute from the original <subMap> element -->
			<xsl:attribute name="value">
				<xsl:value-of select="@value"/>
			</xsl:attribute>
			<!-- Copy the resultMap attribute from the original <subMap> element -->
			<xsl:attribute name="resultMap">
				<xsl:value-of select="@resultMap"/>
			</xsl:attribute>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="/sqlMap/parameterMap">
		<!-- Create the <parameterMap> element -->
		<xsl:element name="parameterMap">
			<!-- Copy the id attribute from the original <parameterMap> element -->
			<xsl:attribute name="id">
				<xsl:value-of select="@id"/>
			</xsl:attribute>
			<!-- Conditionally copy the class attribute if it exists -->
			<xsl:if test="@class">
				<xsl:attribute name="type">
					<xsl:value-of select="@class"/>
				</xsl:attribute>
			</xsl:if>
			<!-- Apply templates to the child elements of <parameterMap> -->
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="/sqlMap/parameterMap/parameter">
		<!-- Copy the entire <parameter> element as is -->
		<xsl:copy-of select="."/>
	</xsl:template>
	
	<xsl:template match="/sqlMap/resultMap/result">
		<!-- Display messages for attributes that cannot be migrated -->
		<xsl:if test="@nullValue">
			<xsl:message>Sorry, I can`t migrate nullValue in: <xsl:copy-of select="."/></xsl:message>
		</xsl:if>
		<xsl:if test="@columnIndex">
			<xsl:message>Sorry, I can`t migrate columnIndex in: <xsl:copy-of select="."/></xsl:message>
		</xsl:if>
		<xsl:if test="@notNullColumn">
			<xsl:message>Sorry, I can`t migrate notNullColumn in: <xsl:copy-of select="."/></xsl:message>
		</xsl:if>
		<!-- Choose between different conditions for the <result> element -->
		<xsl:choose>
			<!-- If the resultMap attribute exists, create a <collection> element -->
			<xsl:when test="@resultMap">
				<xsl:element name="collection">
					<xsl:attribute name="property">
						<xsl:value-of select="@property"/>
					</xsl:attribute>
					<xsl:attribute name="resultMap">
						<xsl:value-of select="@resultMap"/>
					</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<!-- If the select attribute exists, create an <association> element -->
			<xsl:when test="@select">
				<xsl:element name="association">
					<xsl:attribute name="property">
						<xsl:value-of select="@property"/>
					</xsl:attribute>
					<xsl:attribute name="select">
						<xsl:value-of select="@select"/>
					</xsl:attribute>
					<!-- Conditionally copy additional attributes if they exist -->
					<xsl:if test="@column">
						<xsl:attribute name="column">
							<xsl:value-of select="@column"/>
						</xsl:attribute>
					</xsl:if>
					<xsl:if test="@javaType">
						<xsl:attribute name="javaType">
							<xsl:value-of select="@javaType"/>
						</xsl:attribute>
					</xsl:if>
				</xsl:element>
			</xsl:when>
			<!-- Otherwise, create a <result> element -->
			<xsl:otherwise>
				<xsl:element name="result">
					<!-- Conditionally copy the property attribute if it exists -->
					<xsl:if test="@property">
						<xsl:attribute name="property">
							<xsl:value-of select="@property"/>
						</xsl:attribute>
					</xsl:if>
					<!-- Choose between different conditions for the column attribute -->
					<xsl:choose>
						<xsl:when test="@column">
							<xsl:attribute name="column">
								<xsl:value-of select="@column"/>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="column">
								<xsl:value-of select="@property"/>
							</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<!-- Conditionally copy additional attributes if they exist -->
					<xsl:if test="@javaType">
						<xsl:attribute name="javaType">
							<xsl:value-of select="@javaType"/>
						</xsl:attribute>
					</xsl:if>
					<xsl:if test="@jdbcType">
						<xsl:attribute name="jdbcType">
							<xsl:value-of select="@jdbcType"/>
						</xsl:attribute>
					</xsl:if>
					<xsl:if test="@typeHandler">
						<xsl:attribute name="typeHandler">
							<xsl:value-of select="@typeHandler"/>
						</xsl:attribute>
					</xsl:if>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="/sqlMap/sql">
		<!-- Create the <sql> element -->
		<xsl:element name="sql">
			<!-- Copy the id attribute from the original <sql> element -->
			<xsl:attribute name="id">
				<xsl:value-of select="@id"/>
			</xsl:attribute>
			<!-- Apply templates to the child elements of <sql> -->
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>
