<xsl:stylesheet version="3.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs">
    <xsl:output indent="true"></xsl:output>
    <!-- Main template matching the root element -->
    <xsl:template match="/sqlMapConfig">
        <!-- Add a comment with the current date and time -->
        <xsl:comment>Converted at: <xsl:value-of select="current-dateTime()"/> </xsl:comment>
        <xsl:text disable-output-escaping="yes"><![CDATA[
				<!DOCTYPE configuration PUBLIC "-//mybatis.org//DTD SQL Map Config 3.0//EN"
				"http://mybatis.org/dtd/mybatis-3-config.dtd">
				]]></xsl:text>
        
        <xsl:element name="configuration">
            <xsl:apply-templates select="/sqlMapConfig/settings"/>
            <xsl:call-template name="generate-type-aliases"/>
            <xsl:call-template name="generate-type-handlers"/>
            <xsl:call-template name="generate-object-factory"/>
            <xsl:call-template name="generate-environments"/>
            <xsl:call-template name="generate-mappers"/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to generate typeAliases -->
    <xsl:template name="generate-type-aliases">
        <xsl:element name="typeAliases">
            <xsl:for-each select="/sqlMapConfig/typeAlias">
                <xsl:copy-of select="."/>
            </xsl:for-each>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to generate typeHandlers -->
    <xsl:template name="generate-type-handlers">
        <xsl:element name="typeHandlers">
            <xsl:for-each select="/sqlMapConfig/typeHandler">
                <xsl:element name="typeHandler">
                    <xsl:attribute name="javaType">
                        <xsl:value-of select="@javaType"/>
                    </xsl:attribute>
                    <xsl:attribute name="jdbcType">
                        <xsl:value-of select="@jdbcType"/>
                    </xsl:attribute>
                    <xsl:attribute name="handler">
                        <xsl:value-of select="@callback"/>
                    </xsl:attribute>
                </xsl:element>
            </xsl:for-each>
        </xsl:element>
    </xsl:template>
    
    <!-- Template to generate objectFactory -->
    <xsl:template name="generate-object-factory">
        <xsl:if test="resultObjectFactory">
            <xsl:element name="objectFactory">
                <xsl:attribute name="type">
                    <xsl:value-of select="resultObjectFactory/@type"/>
                </xsl:attribute>
            </xsl:element>
        </xsl:if>
    </xsl:template>
    
    <!-- Template to generate environments -->
    <xsl:template name="generate-environments">
        <xsl:if test="transactionManager">
            <xsl:element name="environments">
                <xsl:attribute name="default">default</xsl:attribute>
                <xsl:element name="environment">
                    <xsl:attribute name="id">default</xsl:attribute>
                    <xsl:element name="transactionManager">
                        <xsl:attribute name="type">
                            <xsl:value-of select="transactionManager/@type"/>
                        </xsl:attribute>
                        <xsl:for-each select="transactionManager/property">
                            <xsl:copy-of select="."/>
                        </xsl:for-each>
                    </xsl:element>
                    <xsl:if test="transactionManager/dataSource">
                        <xsl:element name="dataSource">
                            <xsl:attribute name="type">
                                <xsl:value-of select="transactionManager/dataSource/@type"/>
                            </xsl:attribute>
                            <xsl:for-each select="transactionManager/dataSource/property">
                                <xsl:copy-of select="."/>
                            </xsl:for-each>
                        </xsl:element>
                    </xsl:if>
                </xsl:element>
            </xsl:element>
        </xsl:if>
    </xsl:template>
    
    <!-- Template to generate mappers -->
    <xsl:template name="generate-mappers">
        <xsl:element name="mappers">
            <xsl:for-each select="/sqlMapConfig/sqlMap">
                <xsl:element name="mapper">
                    <xsl:copy-of select="@*"/>
                </xsl:element>
            </xsl:for-each>
        </xsl:element>
    </xsl:template>
</xsl:stylesheet>