<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="3.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs">
    <xsl:output method="html" version="5.0" encoding="UTF-8" indent="yes"/>
    
    <!-- Main template that matches the root element -->
    <xsl:template match="/">
        <html xmlns="http://www.w3.org/1999/xhtml">
            <head>
                <title><xsl:value-of select="/rss/channel/title"/> RSS Feed</title>
                <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
            </head>
            <body>
                <div id="content">
                    <p class="explanation">
                        <!--
                            This is a podcast RSS feed generated by <a href="http://www.seriouslysimplepodcasting.com/" target="_blank">Seriously Simple Podcasting</a>. It is meant for consumption by podcast feed readers using the URL in the address bar.
                        -->
                    </p>
                    <xsl:call-template name="channel-header"/>
                    <xsl:call-template name="channel-items"/>
                </div>
            </body>
        </html>
    </xsl:template>
    
    <!-- Template for the channel header -->
    <xsl:template name="channel-header">
        <div id="channel-header">
            <h1>
                <xsl:if test="/rss/channel/image">
                    <div id="channel-image">
                        <a href="{/rss/channel/image/link}">
                            <img src="{/rss/channel/image/url}" title="{/rss/channel/image/title}"/>
                        </a>
                    </div>
                </xsl:if>
                <xsl:value-of select="/rss/channel/title"/>
            </h1>
            <p>
                <xsl:value-of select="/rss/channel/description"/>
            </p>
            <p>
                <a href="{/rss/channel/link}" target="_blank">Visit podcast website &#x0226B;</a>
            </p>
        </div>
    </xsl:template>
    
    <!-- Template for channel items -->
    <xsl:template name="channel-items">
        <xsl:for-each select="/rss/channel/item">
            <div class="channel-item">
                <h2>
                    <a href="{link}" target="_blank">
                        <xsl:value-of select="title"/>
                    </a>
                </h2>
                <xsl:if test="description">
                    <p>
                        <xsl:value-of select="description" disable-output-escaping="yes"/> (<xsl:value-of select="pubDate"/>) 
                    </p>
                </xsl:if>
                <p class="episode_meta">
                    <a href="{enclosure/@url}?ref=download">Download episode</a> | 
                    <a href="{enclosure/@url}?ref=new_window" target="_blank">Play in new window</a> | 
                    File size: <xsl:value-of select="format-number(number(enclosure/@length) div 1024000, '0.0')"/>MB 
                </p>
            </div>
        </xsl:for-each>
    </xsl:template>
</xsl:stylesheet>
